import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Arrays;
import java.util.Random;

/**
 * Třída obalující celočíselné pole hodnot. Poskytuje některé
 * užitečné nástroje pro práci s polem, mimo jiné tisk do konzole se
 * zvýrazněním některých prvků.
 */
public class IntArray {

	/**
	 * Pole hodnot.
	 */
	private int[] data;

	/**
	 * Generátor náhodných hodnot do pole.
	 */
	private static Random generator = new Random();

	/**
	 * Vytvoří nové celočíselné pole náhodných hodnot dané velikosti.
	 * 
	 * @param size
	 *            velikost vytvářeného pole.
	 */
	public IntArray(int size) {
		// alokování pole
		data = new int[size];
		// naplnění pole náhodnými hodnotami
		// z intervalu 10 - 99
		for (int i = 0; i < size; i++) {
			data[i] = 10 + generator.nextInt(90);
		}
	}

	/**
	 * Pomocí knihovní funkce seřadí hodnoty v poli.
	 */
	public void sort() {
		Arrays.sort(data);
	}

	/**
	 * Vrátí hodnotu prvku na daném indexu v poli.
	 * 
	 * @param index
	 *            který prvek vrací.
	 * @return hodnotu prvku na daném indexu.
	 */
	public int get(int index) {
		return data[index];
	}

	/**
	 * Nastaví hodnotu prvku na daném indexu v poli.
	 * 
	 * @param index
	 *            který prvek nastavit.
	 * @param element
	 *            nastavovaná hodnota prvku.
	 */
	public void put(int index, int element) {
		data[index] = element;
	}

	/**
	 * Prohodí mezi sebou prvky na indexech first a second.
	 * 
	 * @param first
	 *            první prvek k prohození.
	 * @param second
	 *            druhý prvek k prohození.
	 */
	public void swap(int first, int second) {
		int temp = data[first];
		data[first] = data[second];
		data[second] = temp;
	}

	/**
	 * Vrátí velikost pole.
	 * 
	 * @return velikost pole.
	 */
	public int size() {
		return data.length;
	}

	/**
	 * Vrátí řetězcovou reprezentaci hodnot v poli. Pouze hodnoty od
	 * indexu from do indexu to budou zobrazeny. Místo ostatních
	 * hodnot se v řetězci vrátí mezery.
	 * 
	 * @param from
	 *            začátek zobrazovaných hodnot (včetně).
	 * @param to
	 *            konec zobrazovaných hodnot (včetně).
	 * @return řetězec s hodnotami prvků mezi indexy from a to.
	 */
	public String toString(int from, int to) {
		StringWriter writer = new StringWriter();
		PrintWriter printer = new PrintWriter(writer);

		for (int i = 0; i < data.length; i++) {
			if (i >= from && i <= to) {
				printer.printf("%3d", data[i]);
			} else {
				printer.print("   ");
			}
		}

		return writer.getBuffer().toString();
	}

	/**
	 * Vrátí řetězcovou reprezentaci všech hodnot v poli.
	 * 
	 * @return řetězcovou reprezentaci pole.
	 */
	public String toString() {
		return toString(0, data.length);
	}

	/**
	 * Vrátí řetězec s hvězdičkou na místě prvku s daným indexem
	 * index. Tuto metodu je možné použít pro zvýraznění prvku pole.
	 * 
	 * @param index
	 *            index prvku, na jehož místě se má zobrazit
	 *            hvězdička.
	 * @return řetězec s hvězdičkou na místě prvku s indexem index.
	 */
	public String starAt(int index) {
		return starAt(index, 0, -1);
	}

	/**
	 * Vrátí řetězec se zvýrazněním prvku na místě index a intervalu
	 * prvků mezi indexy from a to. Prvek na místě index je zvýrazněn
	 * pomocí znaku hvězdičky. Interval prvků mezi indexy from a to
	 * je zvýrazněn pomocí znaků pomlčky.
	 * 
	 * @param index
	 *            index prvku, na jehož místě se má zobrazit
	 *            hvězdička.
	 * @param from
	 *            začátek intervalu, který se má zobrazit pomocí
	 *            pomlček (včetně).
	 * @param to
	 *            konec intervalu, který se má zobrazit pomocí
	 *            pomlček (včetně).
	 * @return řetězec se speciálními znaky na místě daných prvků.
	 */
	public String starAt(int index, int from, int to) {
		StringBuffer sb = new StringBuffer();

		for (int i = 0; i < data.length; i++) {
			if (i >= from && i <= to) {
				if (i == index) {
					sb.append(" *-");
				} else {
					sb.append(" --");
				}
			} else {
				if (i == index) {
					sb.append(" * ");
				} else {
					sb.append("   ");
				}
			}
		}

		return sb.toString();
	}

}